package org.apache.bcel.generic;

import org.apache.bcel.util.ByteSequence;
import java.io.IOException;
import java.io.DataOutputStream;

public abstract class LocalVariableInstruction extends Instruction implements TypedInstruction, IndexedInstruction
{
    @Deprecated
    protected int n;
    private short cTag;
    private short canonTag;
    
    LocalVariableInstruction() {
        this.n = -1;
        this.cTag = -1;
        this.canonTag = -1;
    }
    
    LocalVariableInstruction(final short canonTag, final short cTag) {
        this.n = -1;
        this.cTag = -1;
        this.canonTag = -1;
        this.canonTag = canonTag;
        this.cTag = cTag;
    }
    
    protected LocalVariableInstruction(final short opcode, final short cTag, final int n) {
        super(opcode, (short)2);
        this.n = -1;
        this.cTag = -1;
        this.canonTag = -1;
        this.cTag = cTag;
        this.canonTag = opcode;
        this.setIndex(n);
    }
    
    @Override
    public void dump(final DataOutputStream out) throws IOException {
        if (this.wide()) {
            out.writeByte(196);
        }
        out.writeByte(super.getOpcode());
        if (super.getLength() > 1) {
            if (this.wide()) {
                out.writeShort(this.n);
            }
            else {
                out.writeByte(this.n);
            }
        }
    }
    
    public short getCanonicalTag() {
        return this.canonTag;
    }
    
    @Override
    public final int getIndex() {
        return this.n;
    }
    
    @Override
    public Type getType(final ConstantPoolGen cp) {
        switch (this.canonTag) {
            case 21:
            case 54: {
                return Type.INT;
            }
            case 22:
            case 55: {
                return Type.LONG;
            }
            case 24:
            case 57: {
                return Type.DOUBLE;
            }
            case 23:
            case 56: {
                return Type.FLOAT;
            }
            case 25:
            case 58: {
                return Type.OBJECT;
            }
            default: {
                throw new ClassGenException("Unknown case in switch" + this.canonTag);
            }
        }
    }
    
    @Override
    protected void initFromFile(final ByteSequence bytes, final boolean wide) throws IOException {
        if (wide) {
            this.n = bytes.readUnsignedShort();
            super.setLength(4);
        }
        else {
            final short opcode = super.getOpcode();
            if ((opcode >= 21 && opcode <= 25) || (opcode >= 54 && opcode <= 58)) {
                this.n = bytes.readUnsignedByte();
                super.setLength(2);
            }
            else {
                if (opcode <= 45) {
                    this.n = (opcode - 26) % 4;
                }
                else {
                    this.n = (opcode - 59) % 4;
                }
                super.setLength(1);
            }
        }
    }
    
    @Override
    public void setIndex(final int n) {
        if (n < 0 || n > 65535) {
            throw new ClassGenException("Illegal value: " + n);
        }
        if ((this.n = n) <= 3) {
            super.setOpcode((short)(this.cTag + n));
            super.setLength(1);
        }
        else {
            super.setOpcode(this.canonTag);
            if (this.wide()) {
                super.setLength(4);
            }
            else {
                super.setLength(2);
            }
        }
    }
    
    final void setIndexOnly(final int n) {
        this.n = n;
    }
    
    @Override
    public String toString(final boolean verbose) {
        final short opcode = super.getOpcode();
        if ((opcode >= 26 && opcode <= 45) || (opcode >= 59 && opcode <= 78)) {
            return super.toString(verbose);
        }
        return super.toString(verbose) + " " + this.n;
    }
    
    private boolean wide() {
        return this.n > 255;
    }
}
